<?php
/**
 *
 * An example admin interface for editing the config script.
 * Is a little diffuse, but you could break this up into
 * multiple parts of course. Or rewrite it, to move plugins
 * and their associated settings onto one page for example.
 *
 * If this were a real application, you would add some
 * authorization scheme (HTTP or form login) here.
 *
 */


#-- load admin functions
include_once("./lib/libconfigedit.php");
include_once("./lib/pluginmetadata.php");
include_once("./lib/libconfig.php");
//include_once("./lib/editconfigini.php");


#-- prepare for configuration editing
define("CONFIG_DIR", "./");
define("CONFIG_FILE", "config.php");   // or test.ini
$conf = new generic_config_html(CONFIG_DIR, CONFIG_FILE);


?>
<html>
<head>
  <title>Generic PHP Plugin and Setting Management</title>
  <link rel="stylesheet" href="admin_bright.css" type="text/css" />
  <script type="text/javascript" src="http://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js"></script>
</head>
<body>

<div class="admin-topline">
</div>
<div class="admin-header">
  <h1>Generic PHP Plugin/Setting Management</h1>
</div>


<!-- .............................................................. -->
<div class="admin-menu">

  <a class="head" href="admin_ui.php?cfg=pluginstall"><?php echo img("plugin"); ?>Plugins</a>
  <?php
  foreach ($conf->categories() as $cat) if ($cat && count($conf->plugins($cat))) {
     echo "     <a href=\"admin_ui.php?cfg=plugins&amp;cat=$cat\">".img($cat)."$cat</a>\n";
  }
  ?>

  <a class="head" href="admin_ui.php?cfg=settings&amp;cat="><?php echo img("setting"); ?>Settings</a>
  <?php
  foreach ($conf->categories() as $cat) if ($cat && count($conf->settings($cat))) {
     echo "     <a href=\"admin_ui.php?cfg=settings&amp;cat=$cat\">".img($cat)."$cat</a>\n";
  }
  ?>

  <a class="head" href="."><?php echo img("exit"); ?> Exit</a>

</div>



<!-- .............................................................. -->
<div class="admin-content">

  <?php


  #-- package manager is handled separately
  if ($_REQUEST["cfg"] == "pluginstall") {
     
     include("admin_pluginstall.php");
     pluginstall($conf);
  }
  else {


     #-- form output
     $cfg = $_REQUEST["cfg"];
     $cat = $_REQUEST["cat"];

     $file = "admin_ui.php";
     echo "<form action=\"$file?cfg=$cfg&amp;cat=$cat\" method=\"POST\" enctype=\"multipart/form-data\" accept-encoding=\"ISO-8859-1\">";


     #-- check if writable
     if (!is_writeable($conf->cfg->fn)) {
        echo "<div class=\"message error\">{$conf->cfg->fn} is not writable. <p>Use your shell account or FTP access to 'chmod 666 config.php'. But don't forget to change this back to 644 after you've done. Or get a professional provider which supports suexec/FastCGI instead of the amateurish 'safe_mode'/mod_php setup.</p></div>";
     }

     #-- save if form was submitted
     elseif ($_REQUEST["save"]) {
        $conf->form_receive();
        // saves automatically
     }

  
     #-- show plugins
     if ($cfg == "plugins") {

        #-- headline
        echo "<h2>" . img($cat,20) . " $cat plugins</h2>";

        #-- show plugins
        foreach ($conf->plugins($_REQUEST["cat"]) as $pmd) {
           #-- display entry
           echo $conf->html($pmd);
        }

     }


     #-- just plugin options     
     elseif ($_REQUEST["cfg"] == "settings") {

        #-- headline
        echo "<h2>" . img($cat,20) . " $cat settings</h2>";

        #-- show settings
        foreach ($conf->settings($_REQUEST["cat"]) as $opt) {
           #-- display entry
           echo $conf->html($opt);
        }

     }



     
     #-- start page
     else { ?>
     
        <div class="message">
          <b>Select one of the plugin or setting categories.</b>
          <p>In a real application, you wouldn't leave this admin start page empty, of course.</p>
          <p>Oh, and of course, you'd integrate this interface with your authorization code. It is meant for administration, and not to be accessed by ordinary visitors.</p>
        </div>
     
    <?php } ?>


    <input type="submit" name="save" value="save" title="Save" />
    </form>
    
  <?php

  }#else

  ?>

</div>

</body>
</html>

<?php


#===================================================== utility code ===

#-- return <img> tag for given name
function img($id, $xy=16) {
   $basedir = "./img";
   if (file_exists($fn = "$basedir/$id.png")) {
   }
   elseif ($fn = $list[$id]) {
   }
   else {
      $fn = "$basedir/null.png";
   }
   return "<img src=\"$fn\" align=\"left\" alt=\"$id\" width=\"$xy\" height=\"$xy\" />";
}


?>